"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tasks_1 = require("@angular-devkit/schematics/tasks");
const util_1 = require("../utility/util");
const dependencies_1 = require("../utility/dependencies");
const versions_1 = require("./versions");
const schematics_1 = require("@angular-devkit/schematics");
const change_1 = require("@schematics/angular/utility/change");
const config_1 = require("@schematics/angular/utility/config");
const typescript_1 = require("typescript");
const ng_ast_utils_1 = require("@schematics/angular/utility/ng-ast-utils");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
// You don't have to export the function as default. You can also have more than one rule factory
// per file.
function bootstrapSchematics(_options) {
    let angularJsonVal;
    let project;
    return (tree, _context) => {
        console.log(_options);
        angularJsonVal = getAngularJsonValue(tree);
        project = getProject(angularJsonVal);
        return schematics_1.chain([
            _options.installFontAwesome ? addFontAwesome(project) : schematics_1.noop(),
            updateDependencies(),
            installDependencies(),
            _options.removeStyles ? removeCssFiles() : schematics_1.noop(),
            _options.replaceAppTemplate ? addComponents(_options) : schematics_1.noop(),
            addBootstrapFiles(),
            modifyAngularJson(_options, angularJsonVal, project),
            addModule(_options.project),
        ])(tree, _context);
    };
}
exports.bootstrapSchematics = bootstrapSchematics;
/**
 * Do not specify version since it's in the local package.json file
 * @param project : ng project that schematics run on
 */
function addFontAwesome(project) {
    return (tree, context) => {
        context.logger.debug("Adding fontawesome files");
        return schematics_1.chain([
            schematics_1.externalSchematic('@fortawesome/angular-fontawesome', 'ng-add', {
                project: project
            }),
            schematics_1.mergeWith(schematics_1.apply(schematics_1.url("./fa-files"), [schematics_1.move("./src/app/shared/services")]))
        ])(tree, context);
    };
}
function addModule(projectName) {
    return (tree) => {
        const workspace = config_1.getWorkspace(tree);
        const project = workspace.projects[projectName || workspace.defaultProject];
        const buildOptions = getProjectTargetOptions(project, 'build');
        const modulePath = ng_ast_utils_1.getAppModulePath(tree, buildOptions.main);
        const moduleSource = getSourceFile(tree, modulePath);
        const changes = ast_utils_1.addImportToModule(moduleSource, modulePath, 'CompModule', './comp/comp.module');
        const recorder = tree.beginUpdate(modulePath);
        changes.forEach((change) => {
            if (change instanceof change_1.InsertChange) {
                recorder.insertLeft(change.pos, change.toAdd);
            }
        });
        tree.commitUpdate(recorder);
        return tree;
    };
}
function removeCssFiles() {
    return (tree, context) => {
        context.logger.debug("Removing styles default file");
        if (tree.exists("./src/styles.scss")) {
            tree.delete("./src/styles.scss");
        }
        if (tree.exists("./src/styles.css")) {
            tree.delete("./src/styles.css");
        }
    };
}
function updateDependencies() {
    // let removeDependencies: Observable<Tree>;
    return (tree, context) => {
        context.logger.debug("Updating dependencies...");
        context.addTask(new tasks_1.NodePackageInstallTask());
        dependencies_1.addPackageJsonDependency(tree, {
            type: dependencies_1.NodeDependencyType.Default,
            name: 'bootstrap',
            version: versions_1.bootstrapVersion,
        });
        dependencies_1.addPackageJsonDependency(tree, {
            type: dependencies_1.NodeDependencyType.Default,
            name: '@ng-bootstrap/ng-bootstrap',
            version: versions_1.ngBootstrapVersion,
        });
        dependencies_1.addPackageJsonDependency(tree, {
            type: dependencies_1.NodeDependencyType.Default,
            name: '@angular/localize',
            version: versions_1.ngLocalizeVersion,
        });
        return tree;
    };
}
function installDependencies() {
    return (tree, context) => {
        context.addTask(new tasks_1.NodePackageInstallTask());
        context.logger.debug('✅️ Dependencies installed');
        return tree;
    };
}
function addBootstrapFiles() {
    return (tree, context) => {
        context.logger.debug("Adding bootstrap files");
        return schematics_1.chain([
            schematics_1.mergeWith(schematics_1.apply(schematics_1.url("./assets"), [schematics_1.move("./src/assets")])),
            schematics_1.mergeWith(schematics_1.apply(schematics_1.url("./icon"), [schematics_1.move("./")]))
        ])(tree, context);
    };
}
function getAngularJsonValue(tree) {
    const angularJsonAst = util_1.parseJsonAtPath(tree, "./angular.json");
    return angularJsonAst.value;
}
function getProject(angularJsonValue) {
    return angularJsonValue.defaultProject;
}
function modifyAngularJson(options, angularJsonVal, project) {
    return (tree, context) => {
        if (tree.exists("./angular.json")) {
            // const angularJsonVal = getAngularJsonValue(tree);
            // const project = getProject(angularJsonVal);
            const projectStylesOptionsJson = angularJsonVal["projects"][project]["architect"]["build"]["options"];
            const projectStylesTestJson = angularJsonVal["projects"][project]["architect"]["test"]["options"];
            const styles = [
                "src/assets/vendor.scss", "src/assets/main.scss"
            ];
            if (options.removeStyles) {
                projectStylesOptionsJson["styles"] = styles;
                projectStylesTestJson["styles"] = styles;
            }
            else {
                Array.prototype.push.apply(projectStylesOptionsJson["styles"], styles);
                Array.prototype.push.apply(projectStylesTestJson["styles"], styles);
            }
            context.logger.debug(`Adding bootstrap scss override in angular.json style`);
            return tree.overwrite("./angular.json", JSON.stringify(angularJsonVal, null, 2));
        }
        else {
            throw new schematics_1.SchematicsException("angular.json not found");
        }
    };
}
function addComponents(_options) {
    return () => {
        const rule = applyWithOverwrite(schematics_1.url('./files'), [
            schematics_1.template(Object.assign({}, _options)),
        ]);
        return rule;
    };
}
function applyWithOverwrite(source, rules) {
    return (tree, _context) => {
        const rule = schematics_1.mergeWith(schematics_1.apply(source, [
            ...rules,
            schematics_1.forEach((fileEntry) => {
                if (tree.exists(fileEntry.path)) {
                    tree.overwrite(fileEntry.path, fileEntry.content);
                    return null;
                }
                return fileEntry;
            }),
        ]));
        return rule(tree, _context);
    };
}
function getSourceFile(host, path) {
    const buffer = host.read(path);
    if (!buffer) {
        throw new schematics_1.SchematicsException(`Could not find ${path}.`);
    }
    const content = buffer.toString('utf-8');
    return typescript_1.createSourceFile(path, content, typescript_1.ScriptTarget.Latest, true);
}
function getProjectTargetOptions(project, buildTarget) {
    if (project.targets && project.targets[buildTarget] && project.targets[buildTarget].options) {
        return project.targets[buildTarget].options;
    }
    if (project.architect && project.architect[buildTarget] && project.architect[buildTarget].options) {
        return project.architect[buildTarget].options;
    }
    throw new schematics_1.SchematicsException(`Cannot determine project target configuration for: ${buildTarget}.`);
}
exports.getProjectTargetOptions = getProjectTargetOptions;
//# sourceMappingURL=index.js.map